@@ -30,6 +30,12 @@ test_that("can shortcircuit reduction with done()", {
   expect_identical(out2, out)
 })
 
+test_that("reduce() forces arguments (#643)", {
+  skip_if(!has_force_and_call)
+  compose <- function(f, g) function(x) f(g(x))
+  expect_identical(reduce(list(identity, identity), compose)(1), 1)
+})
+
 
 # accumulate --------------------------------------------------------------
 
@@ -96,6 +102,14 @@ test_that("can terminate accumulate() early with an empty box", {
   expect_equal(accumulate(c("b", "c"), paste2), "b")
 })
 
+test_that("accumulate() forces arguments (#643)", {
+  skip_if(!has_force_and_call)
+  compose <- function(f, g) function(x) f(g(x))
+  fns <- accumulate(list(identity, identity), compose)
+  expect_true(every(fns, function(f) identical(f(1), 1)))
+})
+
+
 # reduce2 -----------------------------------------------------------------
 
 test_that("basic application works", {
@@ -117,6 +131,13 @@ test_that("can shortcircuit reduce2() with done()", {
   expect_identical(out, c("foo", "foo"))
 })
 
+test_that("reduce2() forces arguments (#643)", {
+  skip_if(!has_force_and_call)
+  compose <- function(f, g, ...) function(x) f(g(x))
+  fns <- reduce2(list(identity, identity), "foo", compose)
+  expect_identical(fns(1), 1)
+})
+
 # accumulate2 -------------------------------------------------------------
 
 test_that("basic accumulate2() works", {
@@ -142,6 +163,13 @@ test_that("can terminate accumulate2() early", {
   expect_equal(accumulate2(x, c(".", "-", "."), paste2, .init = "x"), list("x", "x.a", "x.a-b"))
 })
 
+test_that("accumulate2() forces arguments (#643)", {
+  skip_if(!has_force_and_call)
+  compose <- function(f, g, ...) function(x) f(g(x))
+  fns <- accumulate2(list(identity, identity), "foo", compose)
+  expect_true(every(fns, function(f) identical(f(1), 1)))
+})
+
 
 # Life cycle --------------------------------------------------------------
 
